@@ -26,14 +26,11 @@
 #define NAUTILUS_FILE_UTILITIES_H
 
 #include <gio/gio.h>
+#include <gtk/gtk.h>
 
 #define NAUTILUS_SAVED_SEARCH_EXTENSION ".savedSearch"
 #define NAUTILUS_SAVED_SEARCH_MIMETYPE "application/x-gnome-saved-search"
 
-/* Recognizing special file names. */
-gboolean nautilus_file_name_matches_hidden_pattern   (const char *name_or_relative_uri);
-gboolean nautilus_file_name_matches_backup_pattern   (const char *name_or_relative_uri);
-
 /* These functions all return something something that needs to be
  * freed with g_free, is not NULL, and is guaranteed to exist.
  */
@@ -82,6 +79,10 @@ char *   nautilus_get_data_file_path                 (const char *partial_path);
 
 gboolean nautilus_is_file_roller_installed           (void);
 
+/* Inhibit/Uninhibit GNOME Power Manager */
+int    nautilus_inhibit_power_manager                (const char *message) G_GNUC_WARN_UNUSED_RESULT;
+void     nautilus_uninhibit_power_manager            (int cookie);
+
 /* Return an allocated file name that is guranteed to be unique, but
  * tries to make the name readable to users.
  * This isn't race-free, so don't use for security-related things
@@ -98,4 +99,9 @@ nautilus_find_file_insensitive (GFile *parent, const gchar *name);
 
 char * nautilus_get_accel_map_file (void);
 
+GHashTable * nautilus_trashed_files_get_original_directories (GList *files,
+							      GList **unhandled_files);
+void nautilus_restore_files_from_trash (GList *files,
+					GtkWindow *parent_window);
+
 #endif /* NAUTILUS_FILE_UTILITIES_H */
