@@ -40,7 +40,6 @@
 #include "nautilus-icon-private.h"
 #include "nautilus-link.h"
 #include "nautilus-metadata.h"
-#include <eel/eel-background.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
@@ -55,6 +54,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <eel/eel-canvas-rect-ellipse.h>
+#include <libnautilus-private/nautilus-desktop-background.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-changes-queue.h>
 #include <stdio.h>
@@ -70,7 +70,6 @@ static const GtkTargetEntry drop_types [] = {
 	/* prefer "_NETSCAPE_URL" over "text/uri-list" to satisfy web browsers. */
 	{ NAUTILUS_ICON_DND_NETSCAPE_URL_TYPE, 0, NAUTILUS_ICON_DND_NETSCAPE_URL },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
 	{ NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NAUTILUS_ICON_DND_XDNDDIRECTSAVE }, /* XDS Protocol Type */
 	{ NAUTILUS_ICON_DND_RAW_TYPE, 0, NAUTILUS_ICON_DND_RAW },
 	/* Must be last: */
@@ -187,6 +186,10 @@ canvas_rect_world_to_widget (EelCanvas *canvas,
 			     EelIRect *widget_rect)
 {
 	EelDRect window_rect;
+	GtkAdjustment *hadj, *vadj;
+
+	hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (canvas));
+	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (canvas));
 	
 	eel_canvas_world_to_window (canvas,
 				    world_rect->x0, world_rect->y0,
@@ -194,10 +197,10 @@ canvas_rect_world_to_widget (EelCanvas *canvas,
 	eel_canvas_world_to_window (canvas,
 				    world_rect->x1, world_rect->y1,
 				    &window_rect.x1, &window_rect.y1);
-	widget_rect->x0 = (int) window_rect.x0 - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
-	widget_rect->y0 = (int) window_rect.y0 - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
-	widget_rect->x1 = (int) window_rect.x1 - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
-	widget_rect->y1 = (int) window_rect.y1 - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));
+	widget_rect->x0 = (int) window_rect.x0 - gtk_adjustment_get_value (hadj);
+	widget_rect->y0 = (int) window_rect.y0 - gtk_adjustment_get_value (vadj);
+	widget_rect->x1 = (int) window_rect.x1 - gtk_adjustment_get_value (hadj);
+	widget_rect->y1 = (int) window_rect.y1 - gtk_adjustment_get_value (vadj);
 }
 
 static void
@@ -206,8 +209,8 @@ canvas_widget_to_world (EelCanvas *canvas,
 			double *world_x, double *world_y)
 {
 	eel_canvas_window_to_world (canvas,
-				    widget_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas))),
-				    widget_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas))),
+				    widget_x + gtk_adjustment_get_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (canvas))),
+				    widget_y + gtk_adjustment_get_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (canvas))),
 				    world_x, world_y);
 }
 
@@ -359,7 +362,7 @@ nautilus_icon_container_dropped_icon_feedback (GtkWidget *widget,
 	if (dnd_info->shadow != NULL) {
 		/* FIXME bugzilla.gnome.org 42484: 
 		 * Is a destroy really sufficient here? Who does the unref? */
-		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
+		eel_canvas_item_destroy (dnd_info->shadow);
 	}
 
 	/* Build the selection list and the shadow. */
@@ -598,50 +601,6 @@ nautilus_icon_container_selection_items_local (NautilusIconContainer *container,
 	return result;
 }
 
-static GdkDragAction 
-get_background_drag_action (NautilusIconContainer *container, 
-			    GdkDragAction action)
-{
-	/* FIXME: This function is very FMDirectoryView specific, and
-	 * should be moved out of nautilus-icon-dnd.c */
-	GdkDragAction valid_actions;
-
-	if (action == GDK_ACTION_ASK) {
-		valid_actions = NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND;
-		if (!eel_background_is_desktop (eel_get_widget_background (GTK_WIDGET (container)))) {
-			valid_actions |= NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
-		}
-
-		action = nautilus_drag_drop_background_ask 
-			(GTK_WIDGET (container), valid_actions);
-	}
-
-	return action;
-}
-
-/* handle dropped tile images */
-static void
-receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction action, GtkSelectionData *data)
-{
-	g_assert (data != NULL);
-
-	action = get_background_drag_action (container, action);
-
-	if (action > 0) {
-		char *uri;
-
-		uri = get_container_uri (container);
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "dropped tile image on icon container displaying %s", uri);
-		g_free (uri);
-
-		eel_background_receive_dropped_background_image
-			(eel_get_widget_background (GTK_WIDGET (container)), 
-			 action, 
-			 gtk_selection_data_get_data (data));
-	}
-}
-
 /* handle dropped url */
 static void
 receive_dropped_netscape_url (NautilusIconContainer *container, const char *encoded_url, GdkDragContext *context, int x, int y)
@@ -822,54 +781,6 @@ stop_auto_scroll (NautilusIconContainer *container)
 	nautilus_drag_autoscroll_stop (&container->details->dnd_info->drag_info);
 }
 
-static gboolean
-confirm_switch_to_manual_layout (NautilusIconContainer *container)
-{
-#if 0
-	const char *message;
-	const char *detail;
-	GtkDialog *dialog;
-	int response;
-
-	/* FIXME bugzilla.gnome.org 40915: Use of the word "directory"
-	 * makes this FMIconView specific. Move these messages into
-	 * FMIconView so NautilusIconContainer can be used for things
-	 * that are not directories?
-	 */
-	if (nautilus_icon_container_has_stored_icon_positions (container)) {
-		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it? "
-			"This will clobber the stored manual layout.");
-			detail = no_translate("This folder uses automatic layout.");
-		} else {
-			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them? "
-			"This will clobber the stored manual layout.");
-			detail = no_translate("This folder uses automatic layout.");
-		}
-	} else {
-		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it?");
-			detail = no_translate("This folder uses automatic layout.");
-		} else {
-			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them?");
-			detail = no_translate("This folder uses automatic layout.");
-
-		}
-	}
-
-	dialog = eel_show_yes_no_dialog (message, detail, _("Switch to Manual Layout?"),
-					 GTK_STOCK_CANCEL,
-					 GTK_WINDOW (gtk_widget_get_toplevel(GTK_WIDGET(container))));
-
-	response = gtk_dialog_run (dialog);
-	gtk_object_destroy (GTK_OBJECT (dialog));
-
-	return response == GTK_RESPONSE_YES;
-#else
-	return FALSE;
-#endif
-}
-
 static void
 handle_local_move (NautilusIconContainer *container,
 		   double world_x, double world_y)
@@ -883,10 +794,7 @@ handle_local_move (NautilusIconContainer *container,
 	time_t now;
 
 	if (container->details->auto_layout) {
-		if (!confirm_switch_to_manual_layout (container)) {
-			return;
-		}
-		nautilus_icon_container_freeze_icon_positions (container);
+		return;
 	}
 
 	time (&now);
@@ -1126,7 +1034,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 					       GdkDragContext *context,
 					       int x, int y)
 {
-	char *drop_target;
+	char *drop_target, *container_uri;
 	gboolean local_move_only;
 	double world_x, world_y;
 	gboolean icon_hit;
@@ -1149,28 +1057,37 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 			action = GDK_ACTION_MOVE;
 		} else {
 			action = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+			container_uri = get_container_uri (container);
 			
-			if (selection_is_image_file (container->details->dnd_info->drag_info.selection_list)) {
+			if (eel_uri_is_desktop (container_uri) &&
+			    selection_is_image_file (container->details->dnd_info->drag_info.selection_list)) {
 				action |= NAUTILUS_DND_ACTION_SET_AS_BACKGROUND;
 			}
+
+			g_free (container_uri);
 		}
 		real_action = nautilus_drag_drop_action_ask
 			(GTK_WIDGET (container), action);
 	}
 	
 	if (real_action == (GdkDragAction) NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) {
+		NautilusDesktopBackground *background;
+
+		background = nautilus_desktop_background_new (container);
 		selected_item = container->details->dnd_info->drag_info.selection_list->data;
-		eel_background_receive_dropped_background_image
-			(eel_get_widget_background (GTK_WIDGET (container)),
-			 real_action,
-			 selected_item->uri);
+
+		nautilus_desktop_background_receive_dropped_background_image (background,
+									      selected_item->uri);
+
+		g_object_unref (background);
+
 		return;
 	}
 		
 	if (real_action > 0) {
 		eel_canvas_window_to_world (EEL_CANVAS (container),
-					    x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container))),
-					    y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container))),
+					    x + gtk_adjustment_get_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (container))),
+					    y + gtk_adjustment_get_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (container))),
 					    &world_x, &world_y);
 
 		drop_target = nautilus_icon_container_find_drop_target (container, 
@@ -1249,7 +1166,6 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		*action = nautilus_drag_default_drop_action_for_netscape_url (context);
 		break;
 
-	case NAUTILUS_ICON_DND_BGIMAGE:
 	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = gdk_drag_context_get_suggested_action (context);
 		break;
@@ -1333,7 +1249,7 @@ nautilus_icon_container_free_drag_data (NautilusIconContainer *container)
 	dnd_info->drag_info.got_drop_data_type = FALSE;
 
 	if (dnd_info->shadow != NULL) {
-		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
+		eel_canvas_item_destroy (dnd_info->shadow);
 		dnd_info->shadow = NULL;
 	}
 
@@ -1374,42 +1290,21 @@ drag_begin_callback (GtkWidget      *widget,
 		     gpointer        data)
 {
 	NautilusIconContainer *container;
-	GdkScreen *screen;
-	GdkColormap *colormap;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	cairo_surface_t *surface;
 	double x1, y1, x2, y2, winx, winy;
 	int x_offset, y_offset;
 	int start_x, start_y;
-	gboolean use_mask;
 
 	container = NAUTILUS_ICON_CONTAINER (widget);
 
-	screen = gtk_widget_get_screen (widget);
-	colormap = NULL;
-	if (gdk_screen_is_composited (screen)) {
-		colormap = gdk_screen_get_rgba_colormap (screen);
-		if (colormap != NULL) {
-			use_mask = FALSE;
-		}
-	}
-	
-	/* Fall back on using the same colormap as the widget */
-	if (colormap == NULL) {
-		colormap = gtk_widget_get_colormap (widget);		
-		use_mask = TRUE;
-	}
-
-	start_x = container->details->dnd_info->drag_info.start_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
-	start_y = container->details->dnd_info->drag_info.start_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));
+	start_x = container->details->dnd_info->drag_info.start_x +
+		gtk_adjustment_get_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (container)));
+	start_y = container->details->dnd_info->drag_info.start_y +
+		gtk_adjustment_get_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (container)));
 
         /* create a pixmap and mask to drag with */
-        pixmap = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item, &mask, colormap);
-    
-    	/* we want to drag semi-transparent pixbufs, but X is too slow dealing with
-	   stippled masks, so we had to remove the code; this comment is left as a memorial
-	   to it, with the hope that we get it back someday as X Windows improves */
-	
+        surface = nautilus_icon_canvas_item_get_drag_surface (container->details->drag_icon->item);
+
         /* compute the image's offset */
 	eel_canvas_item_get_bounds (EEL_CANVAS_ITEM (container->details->drag_icon->item),
 				    &x1, &y1, &x2, &y2);
@@ -1418,21 +1313,9 @@ drag_begin_callback (GtkWidget      *widget,
         x_offset = start_x - winx;
         y_offset = start_y - winy;
 
-	if (!use_mask && pixmap != NULL) {
-		cairo_t *cr;
-
-		/* If composite works, make the icons partially transparent */
-		cr = gdk_cairo_create (pixmap);
-		cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OUT);
-		cairo_set_source_rgba(cr, 1,0,0,0.35);
-		cairo_paint (cr);
-		cairo_destroy (cr);
-	}
-	
-	gtk_drag_set_icon_pixmap (context,
-				  colormap,
-				  pixmap, (use_mask ? mask : NULL),
-				  x_offset, y_offset);
+        cairo_surface_set_device_offset (surface, -x_offset, -y_offset);
+        gtk_drag_set_icon_surface (context, surface);
+        cairo_surface_destroy (surface);
 }
 
 void
@@ -1455,8 +1338,10 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
 	/* Notice that the event is in bin_window coordinates, because of
            the way the canvas handles events.
 	*/
-	dnd_info->drag_info.start_x = start_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
-	dnd_info->drag_info.start_y = start_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));	
+	dnd_info->drag_info.start_x = start_x -
+		gtk_adjustment_get_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (container)));
+	dnd_info->drag_info.start_y = start_y -
+		gtk_adjustment_get_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (container)));	
 
 	/* start the drag */
 	context = gtk_drag_begin (GTK_WIDGET (container),
@@ -1467,31 +1352,30 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
 }
 
 static gboolean
-drag_highlight_expose (GtkWidget      *widget,
-		       GdkEventExpose *event,
-		       gpointer        data)
+drag_highlight_draw (GtkWidget *widget,
+                     cairo_t   *cr,
+                     gpointer   user_data)
 {
 	gint x, y, width, height;
 	GdkWindow *window;
-	cairo_t *cr;
 	
-	x = gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (widget)));
-	y = gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (widget)));
-	gdk_drawable_get_size (gtk_widget_get_window (widget), &width, &height);
+	x = gtk_adjustment_get_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (widget)));
+	y = gtk_adjustment_get_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (widget)));
 
-	window = gtk_layout_get_bin_window (GTK_LAYOUT (widget));
-	
-	gtk_paint_shadow (gtk_widget_get_style (widget), window,
+        window = gtk_widget_get_window (widget);
+        width = gdk_window_get_width (window);
+        height = gdk_window_get_height (window);
+
+	gtk_paint_shadow (gtk_widget_get_style (widget),
+                          cr,
 			  GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-			  NULL, widget, "dnd",
+			  widget, "dnd",
 			  x, y, width, height);
-  
-	cr = gdk_cairo_create (window);
+
 	cairo_set_line_width (cr, 1.0);
 	cairo_set_source_rgb (cr, 0, 0, 0);
 	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
 	cairo_stroke (cr);
-	cairo_destroy (cr);
 
 	return FALSE;
 }
@@ -1547,8 +1431,8 @@ start_dnd_highlight (GtkWidget *widget)
 
 	if (!dnd_info->highlighted) {
 		dnd_info->highlighted = TRUE;
-		g_signal_connect_after (widget, "expose_event",
-					G_CALLBACK (drag_highlight_expose),
+		g_signal_connect_after (widget, "draw",
+					G_CALLBACK (drag_highlight_draw),
 					NULL);
 		dnd_highlight_queue_redraw (widget);
 	}
@@ -1563,7 +1447,7 @@ stop_dnd_highlight (GtkWidget *widget)
 
 	if (dnd_info->highlighted) {
 		g_signal_handlers_disconnect_by_func (widget,
-						      drag_highlight_expose,
+						      drag_highlight_draw,
 						      NULL);
 		dnd_highlight_queue_redraw (widget);
 		dnd_info->highlighted = FALSE;
@@ -1669,7 +1553,6 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
 		nautilus_icon_container_dropped_icon_feedback (widget, data, x, y);
 		break;
-	case NAUTILUS_ICON_DND_BGIMAGE:	
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_TEXT:
 	case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
@@ -1705,12 +1588,6 @@ drag_data_received_callback (GtkWidget *widget,
 				(NAUTILUS_ICON_CONTAINER (widget),
 				 context, x, y);
 			break;
-		case NAUTILUS_ICON_DND_BGIMAGE:
-			receive_dropped_tile_image
-				(NAUTILUS_ICON_CONTAINER (widget),
-				 gdk_drag_context_get_selected_action (context),
-				 data);
-			break;
 		case NAUTILUS_ICON_DND_NETSCAPE_URL:
 			receive_dropped_netscape_url
 				(NAUTILUS_ICON_CONTAINER (widget),
