@@ -223,9 +223,9 @@ static void
 edit_bookmarks_dialog_reset_signals (gpointer data,
 				     GObject *obj)
 {
-	g_signal_handler_disconnect (GTK_OBJECT (jump_button),
+	g_signal_handler_disconnect (jump_button,
 				     jump_button_signal_id);
-	g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+	g_signal_handler_disconnect (bookmark_list_widget,
 				     row_activated_signal_id);
 	jump_button_signal_id =
 		g_signal_connect (jump_button, "clicked",
@@ -324,7 +324,6 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 	gtk_widget_show (name_field);
 	gtk_box_pack_start (GTK_BOX (gtk_builder_get_object (builder, "bookmark_name_placeholder")),
 			    name_field, TRUE, TRUE, 0);
-	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (name_field), TRUE);
 	
 	gtk_label_set_mnemonic_widget (
 		GTK_LABEL (gtk_builder_get_object (builder, "bookmark_name_label")),
@@ -334,7 +333,6 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 	gtk_widget_show (uri_field);
 	gtk_box_pack_start (GTK_BOX (gtk_builder_get_object (builder, "bookmark_location_placeholder")),
 			    uri_field, TRUE, TRUE, 0);
-	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (uri_field), TRUE);
 
 	gtk_label_set_mnemonic_widget (
 		GTK_LABEL (gtk_builder_get_object (builder, "bookmark_location_label")),
@@ -408,9 +406,9 @@ void
 edit_bookmarks_dialog_set_signals (GObject *undo_manager_source)
 {
 
-	g_signal_handler_disconnect (GTK_OBJECT (jump_button), 
+	g_signal_handler_disconnect (jump_button,
 				     jump_button_signal_id);
-	g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+	g_signal_handler_disconnect (bookmark_list_widget,
 				     row_activated_signal_id);
 
 	jump_button_signal_id =
@@ -568,28 +566,31 @@ open_selected_bookmark (gpointer user_data, GdkScreen *screen)
 	}
 
 	if (NAUTILUS_IS_NAVIGATION_WINDOW (user_data)) {
-		nautilus_window_go_to (NAUTILUS_WINDOW (user_data), location);
+		window = user_data;
 	} else if (NAUTILUS_IS_SPATIAL_WINDOW (user_data)) {
-		window = nautilus_application_present_spatial_window (application, 
-								      NULL,
-								      NULL,
-								      location,
-								      screen);
+		window = nautilus_application_get_spatial_window (application,
+								  NULL,
+								  NULL,
+								  location,
+								  screen,
+								  NULL);
 	} else { /* window that opened bookmarks window has been closed */
-		if (parent_is_browser_window || eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
-			window = nautilus_application_create_navigation_window (application, 
+		if (parent_is_browser_window || g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+			window = nautilus_application_create_navigation_window (application,
 										NULL,
 										screen);
-			nautilus_window_go_to (window, location);
 		} else {
-			window = nautilus_application_present_spatial_window (application, 
-									      NULL,
-									      NULL,
-									      location,
-									      screen);
+			window = nautilus_application_get_spatial_window (application,
+									  NULL,
+									  NULL,
+									  location,
+									  screen,
+									  NULL);
 		}
 	}
 
+	nautilus_window_go_to (window, location);
+
 	g_object_unref (location);
 }
 
@@ -723,7 +724,7 @@ on_key_pressed (GtkTreeView *view,
                 GdkEventKey *event,
                 gpointer user_data)
 {
-        if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete) {
+        if (event->keyval == GDK_KEY_Delete || event->keyval == GDK_KEY_KP_Delete) {
                 bookmarks_delete_bookmark ();
                 return TRUE;
         }
@@ -1053,7 +1054,7 @@ handle_close_accelerator (GtkWindow *window,
 	g_assert (event != NULL);
 	g_assert (user_data == NULL);
 
-	if (eel_gtk_window_event_is_close_accelerator (window, event)) {		
+	if (event->state & GDK_CONTROL_MASK && event->keyval == GDK_KEY_w) {
 		gtk_widget_hide (GTK_WIDGET (window));
 		return TRUE;
 	}
