@@ -33,7 +33,6 @@
 #include "nautilus-location-bar.h"
 #include "nautilus-search-bar.h"
 #include "nautilus-pathbar.h"
-#include "nautilus-main.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
 #include "nautilus-navigation-window-slot.h"
@@ -86,7 +85,9 @@ static void begin_location_change                     (NautilusWindowSlot
 						       GList                      *new_selection,
 						       NautilusLocationChangeType  type,
 						       guint                       distance,
-						       const char                 *scroll_pos);
+						       const char                 *scroll_pos,
+						       NautilusWindowGoToCallback  callback,
+						       gpointer                    user_data);
 static void free_location_change                      (NautilusWindowSlot         *slot);
 static void end_location_change                       (NautilusWindowSlot         *slot);
 static void cancel_location_change                    (NautilusWindowSlot         *slot);
@@ -472,29 +473,33 @@ new_window_show_callback (GtkWidget *widget,
                                               user_data);
 }
 
-
 void
 nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 					 GFile *location,
 					 NautilusWindowOpenMode mode,
 					 NautilusWindowOpenFlags flags,
-					 GList *new_selection)
+					 GList *new_selection,
+					 NautilusWindowGoToCallback callback,
+					 gpointer user_data)
 {
 	NautilusWindow *window;
         NautilusWindow *target_window;
         NautilusWindowPane *pane;
         NautilusWindowSlot *target_slot;
 	NautilusWindowOpenFlags slot_flags;
-        gboolean do_load_location = TRUE;
+        gboolean existing = FALSE;
 	GFile *old_location;
 	char *old_uri, *new_uri;
 	int new_slot_position;
 	GList *l;
+	gboolean target_spatial, target_navigation, target_same;
+	gboolean is_desktop;
 
 	window = slot->pane->window;
 
         target_window = NULL;
 	target_slot = NULL;
+	target_spatial = target_navigation = target_same = FALSE;
 
 	old_uri = nautilus_window_slot_get_location_uri (slot);
 	if (old_uri == NULL) {
@@ -512,70 +517,83 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 	g_assert (!((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) != 0 &&
 		    (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0));
 
+	is_desktop = NAUTILUS_IS_DESKTOP_WINDOW (window);
+	target_same = is_desktop &&
+		!nautilus_desktop_window_loaded (NAUTILUS_DESKTOP_WINDOW (window));
 
 	old_location = nautilus_window_slot_get_location (slot);
+
 	switch (mode) {
         case NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE :
 		if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
-			target_window = window;
-			if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-				if (!NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change) {
-					target_window = nautilus_application_create_navigation_window 
-						(window->application,
-						 NULL,
-						 gtk_window_get_screen (GTK_WINDOW (window)));
-				} else {
-					NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = FALSE;
-				}
-			} else if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) != 0) {
-				target_window = nautilus_application_create_navigation_window 
-					(window->application,
-					 NULL,
-					 gtk_window_get_screen (GTK_WINDOW (window)));
+			/* always use browser: if we're on the desktop the target is a new navigation window,
+			 * otherwise it's the same window.
+			 */
+			if (is_desktop) {
+				target_navigation = TRUE;
+			} else {
+				target_same = TRUE;
 			}
 		} else if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-                        if (!NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change) {
-                                target_window = nautilus_application_present_spatial_window_with_selection (
-                                        window->application,
-					window,
-					NULL,
-                                        location,
-					new_selection,
-                                        gtk_window_get_screen (GTK_WINDOW (window)));
-                                do_load_location = FALSE;
-                        } else {
-                                NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = FALSE;
-                                target_window = window;
-                        }
+			/* don't always use browser: if source is spatial, target is spatial */
+			target_spatial = TRUE;
 		} else if (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) {
-			target_window = nautilus_application_create_navigation_window 
-				(window->application,
-				 NULL,
-				 gtk_window_get_screen (GTK_WINDOW (window)));
-		} else {
-                        target_window = window;
-                }       
+			/* if it's specified to open a new window, and we're not using spatial,
+			 * the target is a navigation.
+			 */
+			target_navigation = TRUE;
+		}
                 break;
         case NAUTILUS_WINDOW_OPEN_IN_SPATIAL :
-                target_window = nautilus_application_present_spatial_window (
-                        window->application,
-			window,
-			NULL,
-                        location,
-                        gtk_window_get_screen (GTK_WINDOW (window)));
-                break;
+		target_spatial = TRUE;
+		break;
         case NAUTILUS_WINDOW_OPEN_IN_NAVIGATION :
-                target_window = nautilus_application_create_navigation_window 
-                        (window->application,
-			 NULL,
-                         gtk_window_get_screen (GTK_WINDOW (window)));
+		target_navigation = TRUE;
                 break;
         default :
-                g_warning ("Unknown open location mode");
+                g_critical ("Unknown open location mode");
 		g_object_unref (old_location);
                 return;
         }
 
+	/* now get/create the window according to the mode */
+	if (target_same) {
+		target_window = window;
+	} else if (target_navigation) {
+		target_window = nautilus_application_create_navigation_window
+			(window->application,
+			 NULL,
+			 gtk_window_get_screen (GTK_WINDOW (window)));
+	} else {
+		target_window = nautilus_application_get_spatial_window
+			(window->application,
+			 window,
+			 NULL,
+			 location,
+			 gtk_window_get_screen (GTK_WINDOW (window)),
+			 &existing);
+	}
+
+	/* if the spatial window is already showing, present it and set the
+	 * new selection, if present.
+	 */
+	if (existing) {
+		target_slot = target_window->details->active_pane->active_slot;
+
+		gtk_window_present (GTK_WINDOW (target_window));
+
+		if (new_selection != NULL && slot->content_view != NULL) {
+			nautilus_view_set_selection (target_slot->content_view, new_selection);
+		}
+
+		/* call the callback successfully */
+		if (callback != NULL) {
+			callback (window, NULL, user_data);
+		}
+
+		return;
+	}
+
         g_assert (target_window != NULL);
 
 	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0 &&
@@ -614,12 +632,14 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		}
 	}
 
-        if ((!do_load_location) ||
-	    (target_window == window && target_slot == slot &&
+        if ((target_window == window && target_slot == slot &&
 	     old_location && g_file_equal (old_location, location))) {
-		if (old_location) {
-			g_object_unref (old_location);
+
+		if (callback != NULL) {
+			callback (window, NULL, user_data);
 		}
+
+		g_object_unref (old_location);
                 return;
         }
 	
@@ -628,7 +648,7 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 	}
 
         begin_location_change (target_slot, location, new_selection,
-                               NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL);
+			       NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL, callback, user_data);
 
 	/* Additionally, load this in all slots that have no location, this means
 	   we load both panes in e.g. a newly opened dual pane window. */
@@ -637,7 +657,7 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		slot = pane->active_slot;
 		if (slot->location == NULL && slot->pending_location == NULL) {
 			begin_location_change (slot, location, new_selection,
-					       NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL);
+					       NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL, NULL, NULL);
 		}
 	}
 }
@@ -656,7 +676,7 @@ nautilus_window_slot_open_location (NautilusWindowSlot *slot,
 	
 	nautilus_window_slot_open_location_full (slot, location,
 						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-						 flags, NULL);
+						 flags, NULL, NULL, NULL);
 }
 
 void
@@ -673,7 +693,7 @@ nautilus_window_slot_open_location_with_selection (NautilusWindowSlot *slot,
 	}
 	nautilus_window_slot_open_location_full (slot, location,
 						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-						 flags, selection);
+						 flags, selection, NULL, NULL);
 }
 
 
@@ -694,7 +714,7 @@ nautilus_window_slot_go_home (NautilusWindowSlot *slot, gboolean new_tab)
 	home = g_file_new_for_path (g_get_home_dir ());
 	nautilus_window_slot_open_location_full (slot, home, 
 						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 
-						 flags, NULL);
+						 flags, NULL, NULL, NULL);
 	g_object_unref (home);
 }
 
@@ -781,9 +801,23 @@ nautilus_window_slot_content_view_matches_iid (NautilusWindowSlot *slot,
 	if (slot->content_view == NULL) {
 		return FALSE;
 	}
-	return eel_strcmp (nautilus_view_get_view_id (slot->content_view), iid) == 0;
+	return g_strcmp0 (nautilus_view_get_view_id (slot->content_view), iid) == 0;
 }
 
+static gboolean
+report_callback (NautilusWindowSlot *slot,
+		 GError *error)
+{
+	if (slot->open_callback != NULL) {
+		slot->open_callback (slot->pane->window, error, slot->open_callback_user_data);
+		slot->open_callback = NULL;
+		slot->open_callback_user_data = NULL;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
 
 /*
  * begin_location_change
@@ -796,6 +830,8 @@ nautilus_window_slot_content_view_matches_iid (NautilusWindowSlot *slot,
  * @distance: If type is back or forward, the index into the back or forward chain. If
  * type is standard or reload, this is ignored, and must be 0.
  * @scroll_pos: The file to scroll to when the location is loaded.
+ * @callback: function to be called when the location is changed.
+ * @user_data: data for @callback.
  *
  * This is the core function for changing the location of a window. Every change to the
  * location begins here.
@@ -806,7 +842,9 @@ begin_location_change (NautilusWindowSlot *slot,
 		       GList *new_selection,
                        NautilusLocationChangeType type,
                        guint distance,
-                       const char *scroll_pos)
+                       const char *scroll_pos,
+		       NautilusWindowGoToCallback callback,
+		       gpointer user_data)
 {
 	NautilusWindow *window;
         NautilusDirectory *directory;
@@ -839,7 +877,10 @@ begin_location_change (NautilusWindowSlot *slot,
 	slot->pending_selection = eel_g_object_list_copy (new_selection);
 
 	slot->pending_scroll_to = g_strdup (scroll_pos);
-        
+
+	slot->open_callback = callback;
+	slot->open_callback_user_data = user_data;
+
         directory = nautilus_directory_get (location);
 
 	/* The code to force a reload is here because if we do it
@@ -1127,22 +1168,25 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 		}
 		create_content_view (slot, view_id);
 		g_free (view_id);
+
+		report_callback (slot, NULL);
 	} else {
-		display_view_selection_failure (window, file,
-						location, error);
+		if (!report_callback (slot, error)) {
+			display_view_selection_failure (window, file,
+							location, error);
+		}
 
 		if (!gtk_widget_get_visible (GTK_WIDGET (window))) {
 			/* Destroy never-had-a-chance-to-be-seen window. This case
 			 * happens when a new window cannot display its initial URI. 
 			 */
 			/* if this is the only window, we don't want to quit, so we redirect it to home */
-			if (nautilus_application_get_n_windows () <= 1) {
-				g_assert (nautilus_application_get_n_windows () == 1);
 
-				/* Make sure we re-use this window */
-				if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-					NAUTILUS_SPATIAL_WINDOW (window)->affect_spatial_window_on_next_location_change = TRUE;
-				}
+			NautilusApplication *app;
+
+			app = nautilus_application_dup_singleton ();
+			
+			if (g_list_length (gtk_application_get_windows (GTK_APPLICATION (app))) == 1) {
 				/* the user could have typed in a home directory that doesn't exist,
 				   in which case going home would cause an infinite loop, so we
 				   better test for that */
@@ -1159,12 +1203,14 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 						g_object_unref (root);
 					}
 				} else {
-					gtk_object_destroy (GTK_OBJECT (window));
+					gtk_widget_destroy (GTK_WIDGET (window));
 				}
 			} else {
 				/* Since this is a window, destroying it will also unref it. */
-				gtk_object_destroy (GTK_OBJECT (window));
+				gtk_widget_destroy (GTK_WIDGET (window));
 			}
+
+			g_object_unref (app);
 		} else {
 			/* Clean up state of already-showing window */
 			end_location_change (slot);
@@ -1228,7 +1274,7 @@ create_content_view (NautilusWindowSlot *slot,
 	} 
         
         if (slot->content_view != NULL &&
-	    eel_strcmp (nautilus_view_get_view_id (slot->content_view),
+	    g_strcmp0 (nautilus_view_get_view_id (slot->content_view),
 			view_id) == 0) {
                 /* reuse existing content view */
                 view = slot->content_view;
@@ -1255,7 +1301,7 @@ create_content_view (NautilusWindowSlot *slot,
 				   FALSE,
 				   TRUE);
 
-		eel_g_object_list_free (slot->pending_selection);
+		g_list_free_full (slot->pending_selection, g_object_unref);
 		slot->pending_selection = NULL;
 	} else if (slot->location != NULL) {
 		selection = nautilus_view_get_selection (slot->content_view);
@@ -1264,7 +1310,7 @@ create_content_view (NautilusWindowSlot *slot,
 				   selection,
 				   FALSE,
 				   TRUE);
-		eel_g_object_list_free (selection);
+		g_list_free_full (selection, g_object_unref);
 	} else {
 		/* Something is busted, there was no location to load.
 		   Just load the homedir. */
@@ -1316,8 +1362,8 @@ load_new_location (NautilusWindowSlot *slot,
 		   report_load_underway was called from load_location */
 		nautilus_view_set_selection (view, selection_copy);
 	}
-	
-        eel_g_object_list_free (selection_copy);
+
+	g_list_free_full (selection_copy, g_object_unref);
 }
 
 /* A view started to load the location its viewing, either due to
@@ -1736,9 +1782,9 @@ free_location_change (NautilusWindowSlot *slot)
 	}
         slot->pending_location = NULL;
 
-	eel_g_object_list_free (slot->pending_selection);
+	g_list_free_full (slot->pending_selection, g_object_unref);
 	slot->pending_selection = NULL;
-	
+
         /* Don't free pending_scroll_to, since thats needed until
          * the load_complete callback.
          */
@@ -1786,7 +1832,7 @@ cancel_location_change (NautilusWindowSlot *slot)
 				   selection,
 				   TRUE,
 				   FALSE);
-		eel_g_object_list_free (selection);
+		g_list_free_full (selection, g_object_unref);
         }
 
         end_location_change (slot);
@@ -1835,7 +1881,7 @@ nautilus_window_report_view_failed (NautilusWindow *window,
 	if (fallback_load_location != NULL) {
 		/* We loose the pending selection change here, but who cares... */
 		begin_location_change (slot, fallback_load_location, NULL,
-				       NAUTILUS_LOCATION_CHANGE_FALLBACK, 0, NULL);
+				       NAUTILUS_LOCATION_CHANGE_FALLBACK, 0, NULL, NULL, NULL);
 		g_object_unref (fallback_load_location);
 	}
 
@@ -2057,7 +2103,7 @@ nautilus_navigation_window_back_or_forward (NautilusNavigationWindow *window,
 		nautilus_window_slot_open_location_full (slot, location,
 							 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 							 NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB,
-							 NULL);
+							 NULL, NULL, NULL);
 	} else {
         	char *scroll_pos;
 		
@@ -2067,7 +2113,8 @@ nautilus_navigation_window_back_or_forward (NautilusNavigationWindow *window,
 			 location, NULL,
 			 back ? NAUTILUS_LOCATION_CHANGE_BACK : NAUTILUS_LOCATION_CHANGE_FORWARD,
 			 distance,
-			 scroll_pos);
+			 scroll_pos,
+			 NULL, NULL);
 
 		g_free (scroll_pos);
 	}
@@ -2101,10 +2148,11 @@ nautilus_window_slot_reload (NautilusWindowSlot *slot)
 	}
 	begin_location_change
 		(slot, location, selection,
-		 NAUTILUS_LOCATION_CHANGE_RELOAD, 0, current_pos);
+		 NAUTILUS_LOCATION_CHANGE_RELOAD, 0, current_pos,
+		 NULL, NULL);
         g_free (current_pos);
 	g_object_unref (location);
-	eel_g_object_list_free (selection);
+	g_list_free_full (selection, g_object_unref);
 }
 
 void
